/*
 * Copyright (c) 2024 ShinGeTsu Meter.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "hardware/i2c.h"

#include "mcp3424.h"

static uint8_t addr;    // 104;
/*
  7    1:start
  6-5  00:ch1
       01:ch2
       10:ch3
       11: ch4
  4    1:ワンショット  0:連続
  3-2  00:240sps   12bit  4200us   2048
       01: 60sps   14bit    17ms   8192
       10: 15sps   16bit    67ms  32768
       11: 3.75sps 18bit   246ms 13107
  1-0  00:等倍
       01:2倍
       10:4倍
       11:8倍
 */
const adcConf ADC_CONF[] = {
    {0b10010000, 2048, 4200},        // 12bit
    {0b10010100, 8192, 16667},       // 14bit
    {0b10011000, 32768, 66667},      // 16bit
    {0b10011100, 131072, 266667},    // 18bit
};

static uint8_t       command = 0;
static uint8_t       BUFF[2];
static ad_resolution last;

static i2c_inst_t *i2c;

void MCP3424_Init(i2c_inst_t *i, uint8_t a) {
    i2c = i;
    addr = a;
}

uint32_t MCP3424_startConversion(ad_channel ch, ad_resolution reso, bool wait) {
    uint32_t result = 0;
    last = reso;
    uint8_t mask = ch << 5;
    command = ADC_CONF[reso].command | mask;
    i2c_write_blocking(i2c, addr, &command, 1, false);
    if (wait) {
        sleep_us(ADC_CONF[reso].wait);
    } else {
        result = ADC_CONF[reso].wait;
    }
    return result;
}

double MCP3424_read() {
    i2c_read_blocking(i2c, addr, BUFF, 2, false);
    int16_t raw = (BUFF[0] << 8) + BUFF[1];
    double  result = raw * 2.048 / ADC_CONF[last].div;    // この式は16ビットまで
    return result;
}